<?php

namespace ShoppingFeed\Manager\Model\Sales\Order;

use ShoppingFeed\Manager\Api\Data\Account\StoreInterface;
use ShoppingFeed\Manager\Model\Config\Field\Checkbox;

class Config extends AbstractConfig implements ConfigInterface
{
    const KEY_USE_ITEM_REFERENCE_AS_PRODUCT_ID = 'use_item_reference_as_product_id';
    const KEY_SYNC_NON_IMPORTED_ADDRESSES = 'sync_non_imported_addresses';
    const KEY_USE_MOBILE_PHONE_NUMBER_FIRST = 'use_mobile_phone_number_first';
    const KEY_CREATE_INVOICE = 'create_invoice';

    public function getScopeSubPath()
    {
        return [ 'general' ];
    }

    protected function getBaseFields()
    {
        return [
            $this->fieldFactory->create(
                Checkbox::TYPE_CODE,
                [
                    'name' => self::KEY_USE_ITEM_REFERENCE_AS_PRODUCT_ID,
                    'isCheckedByDefault' => false,
                    'label' => __('Use Item Reference as Product ID'),
                    'checkedNotice' => __('The item references will be considered to correspond to product IDs.'),
                    'uncheckedNotice' => __(
                        'The item references will be considered to correspond to product SKUs.'
                    ),
                    'sortOrder' => 10,
                ]
            ),

            $this->fieldFactory->create(
                Checkbox::TYPE_CODE,
                [
                    'name' => self::KEY_SYNC_NON_IMPORTED_ADDRESSES,
                    'isCheckedByDefault' => true,
                    'label' => __('Synchronize Addresses of Non-Imported Orders with Shopping Feed'),
                    'sortOrder' => 20,
                ]
            ),

            $this->fieldFactory->create(
                Checkbox::TYPE_CODE,
                [
                    'name' => self::KEY_USE_MOBILE_PHONE_NUMBER_FIRST,
                    'isCheckedByDefault' => true,
                    'label' => __('Use Mobile Phone Number First (If Available)'),
                    'sortOrder' => 30,
                ]
            ),

            $this->fieldFactory->create(
                Checkbox::TYPE_CODE,
                [
                    'name' => self::KEY_CREATE_INVOICE,
                    'isCheckedByDefault' => true,
                    'label' => __('Create Invoice'),
                    'sortOrder' => 40,
                ]
            ),
        ];
    }

    public function shouldUseItemReferenceAsProductId(StoreInterface $store)
    {
        return $this->getFieldValue($store, self::KEY_USE_ITEM_REFERENCE_AS_PRODUCT_ID);
    }

    public function shouldSyncNonImportedAddresses(StoreInterface $store)
    {
        return $this->getFieldValue($store, self::KEY_SYNC_NON_IMPORTED_ADDRESSES);
    }

    public function shouldUseMobilePhoneNumberFirst(StoreInterface $store)
    {
        return $this->getFieldValue($store, self::KEY_USE_MOBILE_PHONE_NUMBER_FIRST);
    }

    public function shouldCreateInvoice(StoreInterface $store)
    {
        return $this->getFieldValue($store, self::KEY_CREATE_INVOICE);
    }

    public function getFieldsetLabel()
    {
        return __('Orders - General');
    }
}
